/* cgi.c - CGI functions
 * Copyright (C) 2001 CryptNET, V. Alex Brennen
 *
 * This file is part of cwebldap.
 *
 * cwebldap is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * cwebldap is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#include "cgi.h"

void print_header(unsigned char *path, unsigned char *search_term)
{
	FILE *header;
	int rslt = 0;
	char line[132];

	if(path == NULL)
	{
		rslt = -1;
	}
	else if(NULL == (header = fopen(path,"r")))
	{
		rslt = -1;
	}

	printf("Content-Type: Text/HTML\n\n");

	if(rslt == -1)
	{
		printf("<HTML><HEAD><TITLE>LDAP Directory Search</TITLE></HEAD><BODY BGCOLOR=#FFFFFF>\n");
		printf("<CENTER>LDAP Directory Search</CENTER>\n");
		if(search_term != NULL)
		{
			printf("<H1>Search Results for: <I>%s</I></H1>\n",search_term);
			printf("<HR SIZE=1>\n");
			printf("<PRE>\n");
		}
	}
	else
	{
		while(!feof(header))
		{
			if(fgets(line,131,header))
				printf("%s",line);
		}
		fclose(header);
	}
	return;
}

void print_footer(unsigned char *path)
{
	FILE *footer;
	int rslt = 0;
	char line[132];

	if(path == NULL)
	{
		rslt = -1;
	}
	else if(NULL == (footer = fopen(path,"r")))
	{
		rslt = -1;
	}
	if(rslt == -1)
	{
		printf("</PRE>\n");
		printf("<HR SIZE=1>\n");
		printf("[ <A HREF=query.html>Back To Search Page</A> ]\n");
		printf("</BODY></HTML>\n");
	}
	else
	{
		while(!feof(footer))
		{
			if(fgets(line,131,footer))
				printf("%s",line);
		}

		fclose(footer);
	}

	return;
}

void do_error_page(char *error,unsigned char *header_path,unsigned char *footer_path)
{
	print_header(header_path,NULL);
	printf("<CENTER><H1>System Error</H1></CENTER>\n");
	printf("The following System Error Occurred:\n");
	printf("%s\n",error);
	print_footer(footer_path);
}

struct name_value_pair_dllst *parse_name_value_pairs(char *data)
{
	struct name_value_pair_dllst *cgidata;
	struct name_value_pair_dllst *current;

	char *dataptr;
        char *name;
        char *value;
        char *value_2;
        char *true_name;
        char *true_value;


	cgidata = (struct name_value_pair_dllst *)malloc(sizeof(struct name_value_pair_dllst));
	if(cgidata == NULL)
	{
		fprintf(stderr,"Malloc call failed: Out of memory.\n");

		return NULL;
	}

	cgidata->prev = NULL;
	cgidata->next = NULL;
	cgidata->name = NULL;
	cgidata->value = NULL;

	current = cgidata;

	dataptr = &data[0];

        name = (char *)strtok(dataptr,"&");
        do
        {
		true_name = &name[0];
		true_value = (char *)memchr(name,'=',strlen(name));
		true_value[0] = '\0';
                true_value++;
                if(true_value == NULL) fprintf(stderr,"Unexpected Error: The value is null.");
		CGIHexToASCII(true_value);

		cgidata->name = (char *)malloc(strlen(true_name)+2);
		strncpy(cgidata->name,true_name,strlen(true_name)+1);
		cgidata->value = (char *)malloc(strlen(true_value)+2);
		strncpy(cgidata->value,true_value,strlen(true_value)+1);
		cgidata->next = (struct name_value_pair_dllst *)malloc(sizeof(struct name_value_pair_dllst));
		if(cgidata->next == NULL)
		{
			fprintf(stderr,"Failed to malloc:  out of memory\n");
			return NULL;
		}
		cgidata->next->prev = cgidata;
		cgidata = cgidata->next;
		cgidata->next = NULL;
		cgidata->name = NULL;
		cgidata->value = NULL;

        } while(name = (char *)strtok('\0',"&"));
	cgidata = cgidata->prev;
	free(cgidata->next);
	cgidata->next=NULL;

	return cgidata;
}

/*
   This code in part came from "C Unleashed", it was written by Chad
   Dixon. ISBN: 0-672-31896-2
*/
void CGIHexToASCII(char *s)
{
        static const char *Hex = "0123456789ABCDEF";
        unsigned int Ascii=0;
        char *p;
        char *Match;
        int Error = 0;

        for(p = s; !Error && *s !='\0'; s++)
        {
                if(*s == '%')
                {
                        s++;
                        if((Match = (char *)strchr(Hex, *s)) != NULL)
                        {
                                Ascii = (unsigned int)(Match - Hex);

                                s++;
                                if((Match = (char *)strchr(Hex, *s)) != NULL)
                                {
                                        Ascii <<= 4;
                                        Ascii |= (unsigned int)(Match - Hex);

                                        *p++ = (char)Ascii;
                                }
                        }
                }
                else if(*s == '+')
                {
                          *p++ = ' ';
                }
                else
                {
                     *p++ = *s;
                }
        }
        *p  = '\0';
}
