/* config.c - Configuration file functions
 * Copyright (C) 2001 CryptNET, V. Alex Brennen
 *
 * This file is part of cwebldap.
 *
 * cwebldap is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * cwebldap is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#include "config.h"

int init_config(struct cwebldap_config *config)
{
	int read_conf = 0;

	/*
	    Start by initializing to defaults.  We'll report errors in config, but
	    if init doens't happen for a config var, we can run with the default.
	*/
        strncpy(config->ldapsrvr_name,"localhost",18);
        strncpy(config->ldapsrvr_port,"389",4);
	config->bind_anon = 1;
	strncpy(config->bind_cn,"cn=root",5);
	strncpy(config->bind_passwd,"badpasswd",10);
	strncpy(config->root,"dn=example,dn=com",18);
	strncpy(config->attrs,"cn");
	config->max_rslts = 50;
	config->max_qtime = NULL;

	config->header_path[0] = '\0';
	config->footer_path[0] = '\0';

        return 0;
}

struct attr_equivs *read_config(struct cwebldap_config *config,struct attr_equivs *equivs)
{
	/* Read and parse cks.conf data. */
        /* Load data into cks_config struct */
	FILE *conf_file;
        char line[201];

        if((conf_file = fopen("/usr/local/etc/cwebldap.conf","r")) == NULL)
        {
        	fprintf(stderr,"cwebldap_config:  Was not able to read /etc/cwebldap.conf file");

		return NULL;
        }

	while(fgets(line,sizeof(line),conf_file) != NULL)
        {
		if( (!(memcmp(line,"#",1) == 0)) && (!(isspace(line[0]))) )
                {
			char *name;
                        char *value;

                        name = (char *)strtok(line," ");
                        value = (char *)strtok('\0',"\0");
			value[strlen(value)-1] = '\0';

                        if(memcmp(name,"ldapsrvr_name",13) == 0)
                        {
        			strncpy(config->ldapsrvr_name,value,16);
        		}
                        else if(memcmp(name,"ldapsrvr_port",13) == 0)
                        {
        			strncpy(config->ldapsrvr_port,value,10);
        		}
			else if(memcmp(name,"bind_anon",9) == 0)
			{
				config->bind_anon = atoi(value);
			}
			else if(memcmp(name,"bind_cn",7) == 0)
			{
				strncpy(config->bind_cn,value,119);
			}
			else if(memcmp(name,"bind_passwd",11) == 0)
			{
				strncpy(config->bind_passwd,value,119);
			}
			else if(memcmp(name,"root",4) == 0)
			{
				strncpy(config->root,value,59);
			}
			else if(memcmp(name,"attrs",5) == 0)
			{
				strncpy(config->attrs,value,119);
			}
			else if(memcmp(name,"max_rslts",9) == 0)
			{
				config->max_rslts = atoi(value);
			}
			else if(memcmp(name,"max_qtime",9) == 0)
			{
				/* OpenLDAP doesn't support this right now.  I don't understand why,
				   but they don't support it so I don't support it.  I'll figgure it
				   out later. The ldap.h header file says go look at mailing list
				   discussions. :(
				*/
			/*	int qtime_tmp = 0;
				long mqtime_tmp = 0;

				qtime_tmp = atoi(value);
				if(qtime_tmp == -1)
				{ */
					config->max_qtime = NULL;
			/*	}
				else
				{
					config->max_qtime = (struct timeval *)malloc(sizeof(struct timeval));
					config->max_qtime->tv_sec = (long)qtime_tmp;
					mqtime_tmp = qtime_tmp *100000;
					config->max_qtime->tv_usec = mqtime_tmp;
				} */
			}
			else if(memcmp(name,"attr_equiv",10) == 0)
			{
				char *tmp_dir_name;
				char *tmp_name;

    				struct attr_equivs *new_equivs = (struct attr_equivs *)malloc(sizeof(struct attr_equivs));
				if(new_equivs == NULL)
				{
					fprintf(stderr,"cwebldap: out of memory malloc call failed.\n");
					return NULL;
				}
				new_equivs->prev = NULL;
				new_equivs->next = NULL;
				tmp_dir_name = value;
				tmp_name = (char *)memchr(value,'=',strlen(value));
				tmp_name[0] = '\0';
				tmp_name++;

				new_equivs->dir_name = (char *)malloc(strlen(tmp_dir_name)+1);
				strncpy(new_equivs->dir_name,tmp_dir_name,strlen(tmp_dir_name));
				new_equivs->name = (char *)malloc(strlen(tmp_name)+1);
				strncpy(new_equivs->name,tmp_name,strlen(tmp_name));

				if(equivs == NULL)
				{
					equivs = new_equivs;
				}
				else
				{
					new_equivs->prev = equivs;
					equivs->next = new_equivs;
					equivs = equivs->next;
					if(equivs == NULL) printf("ret equiv is null\n");
				}
			}
			else if(memcmp(name,"header_path",11) == 0)
			{
				strncpy(config->header_path,value,299);
			}
			else if(memcmp(name,"footer_path",11) == 0)
			{
				strncpy(config->footer_path,value,299);
			}
                        else
                        {
                        	fprintf(stderr,"cwebldap_config:  Rejected config var: %s\n",name);
                        }
                }

        }

        fclose(conf_file);

        return equivs;
}
