/* cwebldap.c - cwebldap main program
 * Copyright (C) 2001 CryptNET, V. Alex Brennen
 *
 * This file is part of cwebldap.
 *
 * cwebldap is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * cwebldap is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#include "cwebldap.h"


int main(void)
{
	struct cwebldap_config	*config;

	/* LDAP Variables */
	LDAP *ld;
	int rslt = 0, i=0;
	LDAPMessage *res, *entry;
	char *dn;
	char *a;
	BerElement	*ptr;
	char **vals;
	char *attrs[2];

	/* CGI Variables */
	char *method;
	long content_length = 0;
	char *content;
	char *val;
	char *val_2;
	struct name_value_pair_dllst *form;
	struct attr_equivs *equivs;
	struct attr_equivs *walk_equivs;

	/* Util Vars */
	char stmt[200];

	/* debug */
	unsigned char debug_val[10];

	/* Init the program */
	form = NULL;
	equivs = NULL;
	config = (struct cwebldap_config *)malloc(sizeof(struct cwebldap_config));
	if(config == NULL)
	{
		fprintf(stderr,"cwebldap: out of memory malloc call failed.\n");
		return -1;
	}
	init_config(config);
	equivs = (struct attr_equivs *)read_config(config,equivs);
	if(equivs == NULL)
	{
		fprintf(stderr,"cwebldap:  Fatal Error: Error reading config file.  No attr_equivs found in config.\n");

		return -1;
	}

	/* Get the Search Info. */
	method = (char *)getenv("REQUEST_METHOD");

	if(method == NULL)
        {
		/* Program is probably being run from command Line. */
		/* Run a test */
		strncpy(debug_val,"Brennen",8);
		content = (char *)malloc(1000);
		strncpy(content,"STYPE=Exact+Match&QUERY=Brennen",999);
		form = parse_name_value_pairs(content);
		val = get_value(form,"QUERY");
		if(val == NULL) do_error_page("val is null\n",config->header_path,config->footer_path);
		/* return 0; */
	}
	else if(strcmp(method,"GET") == 0)
	{
		/* Program should only be called with POST method. */
		do_error_page("Program called with GET method rather than POST",config->header_path,config->footer_path);

		return 0;
	}
	else if(strcmp(method,"POST") == 0)
	{
		/* Valid Search attempt */
		content_length = atoi(getenv("CONTENT_LENGTH"));
		if(content_length < 3)
		{
			do_error_page("Please use at least 3 letters in your search.",config->header_path,config->footer_path);

			return 0;
		}
		else if(content_length > 100)
		{
			do_error_page("Excessive content length detected.  Please use less than 100 characters.",config->header_path,config->footer_path);

			return 0;
		}
		/* Read in the QUERY */
		content = (char *)malloc(content_length+1);
                if(content == NULL)
                {
                        do_error_page("Server was unable to malloc memory.  Server out of memory.",config->header_path,config->footer_path);

			return 0;
		}
                fread(content,1,content_length,stdin);
                content[content_length] = '\0';

		/* Process the CGI Form. */
		form = parse_name_value_pairs(content);
		val = get_value(form,"QUERY");
		if(val == NULL)
		{
			do_error_page("Search value is null.  You must provide a search term.",config->header_path,config->footer_path);

			return 0;
		}
	}
	else
	{
		/* This should never occur.  Must be some type of corruption. */
		do_error_page("There was some type of corruption of the method variable.\n",config->header_path,config->footer_path);

		return 0;
	}

	/* Make the LDAP Connection and do the Search. */
	ld = ldap_init(config->ldapsrvr_name,atoi(config->ldapsrvr_port));

	if(config->bind_anon == 1)
	{
		rslt = ldap_simple_bind(ld,NULL,NULL);
		if(!rslt)
		{
			do_error_page("Fatal Error: Failed to bind to ldap server as anonymous.",config->header_path,config->footer_path);

			return 0;
		}
	}
	else
	{
		rslt = ldap_simple_bind(ld,config->bind_cn,config->bind_passwd);
		if(!rslt)
		{
			do_error_page("Fatal Error: Failed to bind to ldap server.",config->header_path,config->footer_path);

			return 0;
		}
	}

	/* I don't handle these yet.  That will be in 0.0.2. I would do it now, but I want to put some code out! :) */
	attrs[0] = "cn";
	attrs[1] = NULL;

	/* We'll need to modify this to reflect exact versus partial match */
	val_2 = get_value(form,"STYPE");
	if(memcmp(val_2,"Exact",5) == 0)
	{
		snprintf(stmt,200,"(sn=%s)",val);
	}
	else
	{
		snprintf(stmt,200,"(sn~=%s)",val);
	}

	rslt = ldap_search_ext_s(ld,config->root,LDAP_SCOPE_SUBTREE,stmt,NULL,(int)0,
	           NULL,NULL,config->max_qtime,config->max_rslts,&res);

	if(rslt != LDAP_SUCCESS)
	{
		do_error_page("An LDAP error occured: Unable to to complete LDAP Search.",config->header_path,config->footer_path);

		return 0;
	}

	print_header(config->header_path, val);

	for( entry = ldap_first_entry(ld,res); entry != NULL; entry = ldap_next_entry(ld,entry) )
	{
		if(entry == NULL)
		{
			printf("Unexpected LDAP Error: Directory Entry is NULL.\n");
			print_footer(config->footer_path);

			return 0;
		}
		/* We'll store the DN so that we can eventually off to retrieve the full LDAP record. */
		dn = ldap_get_dn(ld,entry);

		a = ldap_first_attribute(ld,entry,&ptr);
		if(a == NULL)
		{
			printf("Unexpected LDAP Error: Attributes appear to be NULL.\n");
			print_footer(config->footer_path);

			return 0;
		}
		while(a != NULL)
		{
			struct attr_equivs *tmp_equivs;
			vals = ldap_get_values(ld,entry,a);
			tmp_equivs = get_attr_equivs(equivs,a);
			for(i=0;vals[i] != NULL; i++)
			{
				if(tmp_equivs != NULL)
				{
					if(tmp_equivs->values == NULL)
					{
						tmp_equivs->values = (struct value_dllst *)malloc(sizeof(struct value_dllst));
						tmp_equivs->values->prev = NULL;
						tmp_equivs->values->next = NULL;
						tmp_equivs->values->value = (char *)malloc(strlen(vals[i])+2);
						strncpy(tmp_equivs->values->value,vals[i],strlen(vals[i])+1);
					}
					else
					{
						tmp_equivs->values->next = (struct value_dllst *)malloc(sizeof(struct value_dllst));
						tmp_equivs->values->next->prev = tmp_equivs->values;
						tmp_equivs->values->next->next = NULL;
						tmp_equivs->values->next->value = vals[i];
						tmp_equivs->values = tmp_equivs->values->next;
					}
				}
			}
			ldap_value_free(vals);
			ldap_memfree(a);

			/* Move on to the next retrieved value */
			a = ldap_next_attribute(ld,entry,ptr);
		}
		if(ptr != NULL) { ber_free(ptr,0); }

		/* walk the list and print the results */
		walk_equivs = (struct attr_equivs *)get_first_attr_equiv(equivs);
		while(walk_equivs != NULL)
		{
			printf("%s:\t",walk_equivs->name);
			walk_equivs->values = get_first_value(walk_equivs->values);
			while(walk_equivs->values != NULL)
			{
				printf("%s\t",walk_equivs->values->value);
				walk_equivs->values = walk_equivs->values->next;
			}
			printf("\n");
			free_equiv_vals(walk_equivs);
			walk_equivs = walk_equivs->next;
		}
		/* end of walk code */

		ldap_memfree(dn);
	}

	/* Close connection to the LDAP Server */
	ldap_unbind(ld);

	print_footer(config->footer_path);

	/* Free Up Memory */
	free_name_value_pair_dllst(form);
	free_attr_equivs(equivs);
	free(config);

	return 0;
}
